@@ -147,7 +147,7 @@ void Alerts::append(DocumentAlert *alert)
     beginInsertRows(QModelIndex(), index, index);
     
     // watch out for when the alert is done: such as when an action is triggered
-    connect(alert, &DocumentAlert::done, [&](int index) {
+    connect(alert, &DocumentAlert::done, [this](int index) {
         this->beginRemoveRows(QModelIndex(), index, index);
         auto item = this->m_alerts.takeAt(index);
         if (item) {
@@ -178,16 +178,16 @@ DocumentAlert *DocumentHandler::externallyModifiedAlert()
 {
     auto alert = new DocumentAlert(i18n("File changed externally"), i18n("You can reload the file or save your changes now"), DocumentAlert::WARNING_LEVEL, Alerts::MODIFIED);
     
-    const auto reloadAction = [&]() {
+    const auto reloadAction = [this]() {
         emit this->loadFile(this->fileUrl());
     };
     
-    const auto autoReloadAction = [&]() {
+    const auto autoReloadAction = [this]() {
         this->setAutoReload(true);
         emit this->loadFile(this->fileUrl());
     };
     
-    alert->setActions({{i18n("Reload"), reloadAction}, {i18n("Auto Reload"), autoReloadAction}, {i18n("Ignore"), [&]() {}}});
+    alert->setActions({{i18n("Reload"), reloadAction}, {i18n("Auto Reload"), autoReloadAction}, {i18n("Ignore"), []() {}}});
     return alert;
 }
 
@@ -195,7 +195,7 @@ DocumentAlert *DocumentHandler::canNotSaveAlert(const QString &details)
 {
     auto alert = new DocumentAlert(i18n("File can not be saved"), details, DocumentAlert::DANGER_LEVEL, Alerts::SAVE_ERROR);
     
-    alert->setActions({{i18n("Ignore"), [&]() {}}});
+    alert->setActions({{i18n("Ignore"), []() {}}});
     return alert;
 }
 
@@ -203,7 +203,7 @@ DocumentAlert *DocumentHandler::missingAlert()
 {
     auto alert = new DocumentAlert(i18n("Your file was removed"), i18n("This file does not longer exists in your local storage, however you can save it again"), DocumentAlert::DANGER_LEVEL, Alerts::MISSING);
     
-    const auto saveAction = [&]() {
+    const auto saveAction = [this]() {
         this->saveAs(this->fileUrl());
     };
     
@@ -229,7 +229,7 @@ DocumentHandler::DocumentHandler(QObject *parent)
         m_loader->moveToThread(&m_worker);
         connect(&m_worker, &QThread::finished, m_loader, &QObject::deleteLater);
         connect(this, &DocumentHandler::loadFile, m_loader, &FileLoader::loadFile);
-        connect(m_loader, &FileLoader::fileReady, [&](QString array, QUrl url) {
+        connect(m_loader, &FileLoader::fileReady, [this](QString array, QUrl url) {
             this->setText(array);
             
             if (this->textDocument()) {
@@ -258,11 +258,11 @@ DocumentHandler::DocumentHandler(QObject *parent)
     if (m_autoSave)
         m_autoSaveTimer.start(AUTOSAVE_TIMEOUT);
     
-    connect(this, &DocumentHandler::cursorPositionChanged, [&]() {
+    connect(this, &DocumentHandler::cursorPositionChanged, [this]() {
         emit this->currentLineIndexChanged();
     });
     
-    connect(this->m_watcher, &QFileSystemWatcher::fileChanged, [&](QString url) {
+    connect(this->m_watcher, &QFileSystemWatcher::fileChanged, [this](QString url) {
         if (this->fileUrl() == QUrl::fromLocalFile(url)) {
             // THE FILE WAS REMOVED
             if (!FMH::fileExists(this->fileUrl())) {
@@ -414,7 +414,9 @@ void DocumentHandler::refreshAllBlocks()
 {
     if (textDocument()) {
         for (QTextBlock it = textDocument()->begin(); it != textDocument()->end(); it = it.next())
-            textDocument()->documentLayout()->updateBlock(it);
+        {
+            emit this->textDocument()->documentLayout()->updateBlock(it);
+        }
     }
 }
 
@@ -469,6 +471,14 @@ void DocumentHandler::setDocument(QQuickTextDocument *document)
         this->textDocument()->setModified(false);
         connect(this->textDocument(), &QTextDocument::modificationChanged, this, &DocumentHandler::modifiedChanged);
         
+        connect(this->textDocument(), &QTextDocument::blockCountChanged, this, &DocumentHandler::lineCountChanged);
+        
+        //          connect(this->textDocument(), &QTextDocument::cursorPositionChanged, [this](const QTextCursor &)
+        //                 {
+        //                     qDebug() << "Cursors position changed";
+        //                     emit currentLineIndexChanged();
+        //                 });
+        
         this->load(m_fileUrl);
         
         QTextOption textOptions = this->textDocument()->defaultTextOption();
@@ -483,12 +493,13 @@ int DocumentHandler::cursorPosition() const
 }
 
 void DocumentHandler::setCursorPosition(int position)
-{
-    if (position == m_cursorPosition)
+{    
+    if(m_cursorPosition == position)
+    {
         return;
+    }
     
     m_cursorPosition = position;
-    //     reset();
     emit cursorPositionChanged();
 }
 
@@ -1062,7 +1073,7 @@ void DocumentHandler::toggleFold(const int &line)
         const auto endBlock =
         m_highlighter->findFoldingRegionEnd(startBlock).next();
         
-        
+        qDebug() << "Fold line"<< line << startBlock.position() << endBlock.position() << doc->blockCount();
         // fold
         auto block = startBlock.next();
         while (block.isValid() && block != endBlock) 
@@ -1073,13 +1084,18 @@ void DocumentHandler::toggleFold(const int &line)
         }
         
         
+        for (QTextBlock it = startBlock; it != endBlock; it = it.next())
+        {
+            emit this->textDocument()->documentLayout()->updateBlock(it);
+        }
+        
         // redraw document
-        doc->markContentsDirty(
-            startBlock.position(), endBlock.position() - startBlock.position() + 1);
+        //         doc->markContentsDirty(startBlock.position(), endBlock.position());
+        qDebug() << "Fold line"<< line << startBlock.position() << endBlock.position() << doc->blockCount();
         
         //         // update scrollbars
-        //         emit doc->documentLayout()->documentSizeChanged(
-        //             doc->documentLayout()->documentSize());
+        emit doc->documentLayout()->documentSizeChanged(
+            doc->documentLayout()->documentSize());
     }
 }
 
@@ -1091,12 +1107,30 @@ int DocumentHandler::lineHeight(const int &line)
         return 0;
     }
     
-    return int(doc->documentLayout()->blockBoundingRect(doc->findBlockByNumber(line)).height());
+    return int(doc->documentLayout()->blockBoundingRect(doc->findBlockByLineNumber(line)).height());
+}
+
+int DocumentHandler::lineCount()
+{
+    if (!this->textDocument())
+        return 0;
+    return this->textDocument()->blockCount();
 }
 
 int DocumentHandler::getCurrentLineIndex()
+{    
+    if (!this->textDocument())
+        return -1;
+    
+    return this->textDocument()->findBlock(m_cursorPosition).blockNumber();
+}
+
+int DocumentHandler::goToLine(const int& line)
 {
-    return textCursor().blockNumber();
+    if (!this->textDocument())
+        return this->cursorPosition();
+    const auto block = this->textDocument()->findBlockByLineNumber(line);
+    return block.position() + block.length()-1;     
 }
 
 void DocumentHandler::setEnableSyntaxHighlighting(const bool &value)
